package com.qmth.distributed.print.task.templete.importData;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.distributed.print.business.entity.TBTask;
import com.qmth.distributed.print.business.enums.TaskStatusEnum;
import com.qmth.distributed.print.business.enums.UploadFileEnum;
import com.qmth.distributed.print.business.service.TBTaskService;
import com.qmth.distributed.print.business.util.OssUtil;
import com.qmth.distributed.print.common.contant.SpringContextHolder;
import com.qmth.distributed.print.common.contant.SystemConstant;
import com.qmth.distributed.print.common.util.Result;
import com.qmth.distributed.print.common.util.ResultUtil;
import org.apache.commons.io.FileUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.scheduling.annotation.Async;

import java.io.*;
import java.nio.charset.StandardCharsets;
import java.util.Date;
import java.util.Map;
import java.util.Objects;
import java.util.StringJoiner;

/**
 * @Description: 异步导入模版
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/3/29
 */
public abstract class AsyncImportTaskTemplete {
    private final static Logger log = LoggerFactory.getLogger(AsyncImportTaskTemplete.class);

    public static final String BEGIN_TITLE = "->开始准备处理导入的";
    public static final String EXCEPTION_TITLE = "->数据处理发生异常!";
    public static final String EXCEPTION_DATA = "错误信息:";
    public static final String FINISH_TITLE = "->数据处理结束，共处理了";
    public static final String FINISH_SIZE = "条数据";
    public static final String TXT_PREFIX = ".txt";
    public static final String FORMAT_TIME = DateUtil.format(new Date(), SystemConstant.DEFAULT_DATE_PATTERN);

    /**
     * 异步导入任务
     *
     * @param map
     * @return
     * @throws IOException
     */
    @Async
    public abstract Result importTask(Map<String, Object> map) throws IOException;

    /**
     * 获取上传的文件
     *
     * @param tbTask
     * @return
     * @throws IOException
     */
    public InputStream getUploadFileInputStream(TBTask tbTask) throws IOException {
        OssUtil ossUtil = SpringContextHolder.getBean(OssUtil.class);
        JSONObject jsonObject = JSONObject.parseObject(tbTask.getImportFilePath());
        String path = (String) jsonObject.get(SystemConstant.PATH);
        String type = (String) jsonObject.get(SystemConstant.TYPE);
        InputStream inputStream = null;
        if (Objects.equals(type, SystemConstant.OSS)) {
            byte[] data = ossUtil.ossDownload(path);
            inputStream = new ByteArrayInputStream(data);
        } else {
            StringJoiner localPath = new StringJoiner("").add(SystemConstant.TEMP_FILES_DIR).add(File.separator).add(path);
            inputStream = new FileInputStream(new File(localPath.toString()));
        }
        return inputStream;
    }

    /**
     * 创建txt文件
     *
     * @param tbTask
     */
    public void createTxt(TBTask tbTask) {
        OssUtil ossUtil = SpringContextHolder.getBean(OssUtil.class);
        TBTaskService tbTaskService = SpringContextHolder.getBean(TBTaskService.class);
        ByteArrayOutputStream out = null;
        InputStream inputStream = null;
        try {
            JSONObject jsonObject = JSONObject.parseObject(tbTask.getImportFilePath());
            String path = (String) jsonObject.get(SystemConstant.PATH);
            String type = (String) jsonObject.get(SystemConstant.TYPE);

            out = new ByteArrayOutputStream();
            out.write(tbTask.getSummary().getBytes(StandardCharsets.UTF_8));
            byte[] bookByteAry = out.toByteArray();
            inputStream = new ByteArrayInputStream(bookByteAry);
            StringJoiner stringJoiner = new StringJoiner("");
            path = path.substring(0, path.lastIndexOf(File.separator) + 1);

            stringJoiner.add(path).add(SystemConstant.getUuid()).add(TXT_PREFIX).toString();
            if (Objects.equals(type, SystemConstant.OSS)) {//上传至oss
                ossUtil.ossUpload(stringJoiner.toString(), inputStream, null);
            } else {//上传至服务器
                File finalFile = new File(stringJoiner.toString());
                if (!finalFile.exists()) {
                    finalFile.getParentFile().mkdirs();
                    finalFile.createNewFile();
                }
                FileUtils.copyInputStreamToFile(inputStream, finalFile);
            }
            JSONObject json = new JSONObject();
            json.put(SystemConstant.PATH, stringJoiner.toString());
            json.put(SystemConstant.TYPE, type);
            json.put(SystemConstant.UPLOAD_TYPE, UploadFileEnum.FILE);

            tbTask.setStatus(TaskStatusEnum.FINISH);
            tbTask.setReportFilePath(json.toJSONString());
            tbTaskService.updateById(tbTask);
        } catch (Exception e) {
            log.error("请求出错", e);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        } finally {
            try {
                if (Objects.nonNull(inputStream)) {
                    inputStream.close();
                }
                if (Objects.nonNull(out)) {
                    out.flush();
                    out.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
}
