package com.qmth.distributed.print.api;


import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.distributed.print.business.bean.ApiUser;
import com.qmth.distributed.print.business.bean.dto.PrivilegeDto;
import com.qmth.distributed.print.business.entity.SysPrivilege;
import com.qmth.distributed.print.business.service.SysPrivilegeService;
import com.qmth.distributed.print.business.util.RequestUtils;
import com.qmth.distributed.print.common.util.Result;
import com.qmth.distributed.print.common.util.ResultUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * <p>
 * 菜单权限表 前端控制器
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Api(tags = "菜单权限Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/${prefix.url.sys}/privilege")
@Aac(auth = BOOL.FALSE)
public class SysPrivilegeController {

    @Autowired
    private SysPrivilegeService sysPrivilegeService;

    /**
     * 查询
     *
     * @param request
     * @return
     */
    @ApiOperation(value = "查询")
    @RequestMapping(value = "/list", method = RequestMethod.POST)
    public Result list(HttpServletRequest request) {
        ApiUser apiUser = RequestUtils.getApiUser(request);
        List<PrivilegeDto> privilegeDtoList = sysPrivilegeService.listOrgTree(apiUser.getSchoolId());
        return ResultUtil.ok(privilegeDtoList);
    }

    /**
     * 新增/修改
     *
     * @param request
     * @param privilege
     * @return
     */
    @ApiOperation(value = "新增/修改")
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    public Result save(HttpServletRequest request, @RequestBody SysPrivilege privilege) {
        ApiUser apiUser = RequestUtils.getApiUser(request);
        boolean isSuccess = sysPrivilegeService.savePrivilege(privilege);
        return ResultUtil.ok(isSuccess);
    }

    /**
     * 删除
     *
     * @param privilege
     * @return
     */
    @ApiOperation(value = "删除")
    @RequestMapping(value = "/remove", method = RequestMethod.POST)
    public Result remove(@RequestBody SysPrivilege privilege) {
        boolean isSuccess = sysPrivilegeService.remove(privilege.getId());
        return ResultUtil.ok(isSuccess);
    }

    /**
     * 角色已绑定权限列表
     *
     * @param roleId
     * @return
     */
    @ApiOperation(value = "角色已绑定权限列表")
    @RequestMapping(value = "/get_role_privileges", method = RequestMethod.POST)
    public Result getRolePrivileges(@RequestParam(value = "roleId", required = true) Long roleId) {
        List<String> list = sysPrivilegeService.getRolePrivileges(roleId);
        return ResultUtil.ok(list);
    }
}

